<?php

namespace Drupal\paragraphs_clipboard;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\paragraphs\Entity\Paragraph;

/**
 * Service for handling clipboard operations for Paragraphs entities.
 */
class ParagraphsClipboardService {

  /**
   * The temp store factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Service constructor.
   *
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The temp store factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(PrivateTempStoreFactory $temp_store_factory, EntityTypeManagerInterface $entity_type_manager) {
    $this->tempStoreFactory = $temp_store_factory->get('paragraphs_clipboard');
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Retrieves clipboard data.
   *
   * @return \Drupal\Core\TempStore\PrivateTempStoreFactory
   *   An array containing the 'paragraph_id' and 'revision_id'.
   */
  public function getClipboardData() {
    return $this->tempStoreFactory->get('copy_clipboard');
  }

  /**
   * Stores paragraph ID and revision ID to clipboard.
   *
   * @param int $paragraph_id
   *   The ID of the paragraph entity.
   * @param int $revision_id
   *   The revision ID of the paragraph entity.
   *
   * @return void
   *   Void.
   */
  public function setClipboardData($paragraph_id, $revision_id) {
    $this->tempStoreFactory->delete('copy_clipboard');
    $this->tempStoreFactory->set('copy_clipboard', [
      'paragraph_id' => $paragraph_id,
      'revision_id' => $revision_id,
    ]);
  }

  /**
   * Loads a paragraph revision.
   *
   * @param int $revision_id
   *   The ID of the paragraph revision to load.
   *
   * @return mixed
   *   The loaded paragraph entity object, or NULL if it cannot be loaded.
   */
  public function loadParagraphRevision($revision_id) {
    return $this->entityTypeManager->getStorage('paragraph')->loadRevision($revision_id);
  }

  /**
   * Access check for the "Paste from Clipboard" button.
   *
   * @param \Drupal\paragraphs\Entity\Paragraph $paragraph
   *   The paragraph entity.
   * @param int $real_item_count
   *   The current count of items.
   * @param int $cardinality
   *   The cardinality of the field.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition where the paragraph would be pasted.
   *
   * @return bool
   *   TRUE if access is allowed, FALSE otherwise.
   */
  public function pasteClipboardButtonAccess(Paragraph $paragraph, $real_item_count, $cardinality, $field_definition) {
    if (!$paragraph->access('update')) {
      return FALSE;
    }

    if (!$this->isAllowedParagraphType($paragraph, $field_definition)) {
      return FALSE;
    }

    if ($real_item_count >= $cardinality && $cardinality != FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Checks if the paragraph type in the clipboard is allowed for a specific field.
   *
   * @param \Drupal\paragraphs\Entity\Paragraph $paragraph
   *   The paragraph entity from the clipboard.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition where the paragraph would be pasted.
   *
   * @return bool
   *   TRUE if the paragraph type is allowed for the field, FALSE otherwise.
   */
  public function isAllowedParagraphType(Paragraph $paragraph, $field_definition) {
    $allowed_bundles = $field_definition->getSetting('handler_settings')['target_bundles'] ?? [];
    $clipboard_paragraph_bundle = $paragraph->bundle();

    return in_array($clipboard_paragraph_bundle, $allowed_bundles, TRUE);
  }

}
